/***************************************************************************
                          qsqlite.cpp  -  description
                             -------------------
    begin                : Sat Feb 16 2002
    copyright            : (C) 2002 by Bernd Lachner
    email                : dev@lachner-net.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include "qsqlite.h"

QSQLiteException::QSQLiteException(int iSQLiteErrorCode)
: m_iSQLiteErrorCode(iSQLiteErrorCode)
{
}

QSQLiteException::QSQLiteException(QString qsMessage)
: m_qsMessage(qsMessage)
{
}

QString QSQLiteException::getMessage()
{
  if (!m_qsMessage.isNull())
  {
    return m_qsMessage;
  }
  else
  {
    QString qsMessage = QObject::tr("SQLite exec error: ");
    QString qsErrorCode;
    qsErrorCode = qsErrorCode.arg(m_iSQLiteErrorCode);
    return qsMessage + qsErrorCode;
  }
}

QSQLite::QSQLite(QString qsDatabaseName)
: m_eMode(NORMAL),
  m_pSQL(0)
{
  Open(qsDatabaseName);
}

void QSQLite::Open(QString qsDatabaseName)
{
  m_pSQL = sqlite_open(qsDatabaseName, 0, 0);
  if (m_pSQL == 0)
    throw new QSQLiteException(tr("Can't open database!"));
}

QSQLite::~QSQLite()
{
  Close();
}

void QSQLite::Close()
{
  if (m_pSQL != 0)
  {
    sqlite_close(m_pSQL);
    m_pSQL = 0;
  }
}

int QSQLite::SQLStaticCallback(void *pObject, int iColumns, char** aszData, char** aszColumnNames)
{
	return reinterpret_cast<QSQLite*>(pObject)->SQLCallback(iColumns, aszData, aszColumnNames);
}

int QSQLite::SQLCallback(int iColumns, char** aszData, char** aszColumnNames)
{
	if (m_eMode == COUNT)
  {
		QString qsTemp(aszData[0]);
		m_iCount = qsTemp.toInt();
	}
	else if (m_eMode == ISINTABLE)
	{
		m_bIsInTable = true;
	}
	else
	{
		RowData(iColumns, aszData, aszColumnNames);
	}
	return 0;
}

void QSQLite::Select(QObject *poReceiver, const char *pszMember, QString qsTabName, QString qsColumns, QString qsWhere, QString qsOrderColumn, bool bDesc)
{
  connect(this, SIGNAL(RowData(int, char**, char**)), poReceiver, pszMember);
  QString qsSQL("SELECT ");
  qsSQL += qsColumns;
  qsSQL += " FROM ";
  qsSQL += qsTabName;
  if (qsWhere.length() != 0)
  {
    qsSQL += " WHERE ";
    qsSQL += qsWhere;
  }
  if (qsOrderColumn.length() != 0)
  {
    qsSQL += " ORDER BY ";
    qsSQL += qsOrderColumn;
    if (bDesc == true)
    {
      qsSQL += " DESC";
    }
  }
  int iResult = sqlite_exec(m_pSQL, qsSQL, SQLStaticCallback, this, 0);
  disconnect(this, SIGNAL(RowData(int, char**, char**)), poReceiver, pszMember);  
  if (iResult != 0)
    throw new QSQLiteException(iResult);
}


void QSQLite::SelectDistinct(QObject *poReceiver, const char *pszMember, QString qsTabName, QString qsColumns, QString qsWhere)
{
  connect(this, SIGNAL(RowData(int, char**, char**)), poReceiver, pszMember);
	QString qsSQL("SELECT DISTINCT ");
	qsSQL += qsColumns;
	qsSQL += " FROM ";
	qsSQL += qsTabName;
	if (qsWhere.length() != 0)
	{
		qsSQL += " WHERE ";
		qsSQL += qsWhere;
	}
  int iResult = sqlite_exec(m_pSQL, qsSQL, SQLStaticCallback, this, 0);
  disconnect(this, SIGNAL(RowData(int, char**, char**)), poReceiver, pszMember);
  if (iResult != 0)
    throw new QSQLiteException(iResult);
}


void QSQLite::Insert(QString qsTabName, QStringList &qslistValues, QString qsColumns)
{
	QString qsSQL("INSERT INTO ");
	qsSQL += qsTabName;
  qsSQL += " (";
	qsSQL += qsColumns;
	qsSQL += ") VALUES ( ";
  for ( QStringList::Iterator it = qslistValues.begin(); it != qslistValues.end(); ++it )
  {
    qsSQL += MakeCharVal(*it);
    qsSQL += ",";
  }
  qsSQL.truncate(qsSQL.length()-1);
  qsSQL += ")";
  int iResult = sqlite_exec(m_pSQL, qsSQL, SQLStaticCallback, this, 0);
  if (iResult != 0)
    throw new QSQLiteException(iResult);
}

void QSQLite::ExecSQL(QString qsSQL)
{
  int iResult = sqlite_exec(m_pSQL, qsSQL, SQLStaticCallback, this, 0);
  if (iResult != 0)
    throw new QSQLiteException(iResult);
}

QString QSQLite::MakeCharVal(QString qsVal)
{
	QString qsCharVal("'");
	qsCharVal += qsVal;
	qsCharVal += "'";
  return qsCharVal;
}
 
void QSQLite::Delete(QString qsTabName, QString qsColumn, QString qsValue)
{
  QString qsSQL("DELETE FROM ");
  qsSQL += qsTabName;
  qsSQL += " WHERE ";
  qsSQL += qsColumn;
  qsSQL += " = ";
  qsSQL += MakeCharVal(qsValue);
  int iResult = sqlite_exec(m_pSQL, qsSQL, SQLStaticCallback, this, 0);
  if (iResult != 0)
    throw new QSQLiteException(iResult);
}

void QSQLite::DeleteAll(QString qsTabName)
{
  QString qsSQL("DELETE FROM ");
  qsSQL += qsTabName;
  int iResult = sqlite_exec(m_pSQL, qsSQL, SQLStaticCallback, this, 0);
  if (iResult != 0)
    throw new QSQLiteException(iResult);
}


void QSQLite::DeleteIn(QString qsTabName, QString qsColumn, QString qsSubSelect)
{
  QString qsSQL("DELETE FROM ");
  qsSQL += qsTabName;
  qsSQL += " WHERE ";
  qsSQL += qsColumn;
  qsSQL += " IN ";
  qsSQL += "(" + qsSubSelect + ")";
  int iResult = sqlite_exec(m_pSQL, qsSQL, SQLStaticCallback, this, 0);
  if (iResult != 0)
    throw new QSQLiteException(iResult);
}


void QSQLite::Delete(QString qsTabName, QString qsColumn1, QString qsValue1, QString qsColumn2, QString qsValue2, QString qsCombination)
{
  QString qsSQL("DELETE FROM ");
  qsSQL += qsTabName;
  qsSQL += " WHERE ";
  qsSQL += qsColumn1;
  qsSQL += " = ";
  qsSQL += qsValue1;
  qsSQL += " ";
  qsSQL += qsCombination;
  qsSQL += " ";
  qsSQL += qsColumn2;
  qsSQL += " = ";
  qsSQL += qsValue2;
  int iResult = sqlite_exec(m_pSQL, qsSQL, SQLStaticCallback, this, 0);
  if (iResult != 0)
    throw new QSQLiteException(iResult);
}
	
int QSQLite::Count(QString qsTabName, QString qsColumn, QString qsValue)
{
	m_eMode = COUNT;
	QString qsSQL("SELECT COUNT(*) FROM ");
	qsSQL += qsTabName;
	qsSQL += " WHERE ";
	qsSQL += qsColumn;
	qsSQL += " = ";
	qsSQL += qsValue;
  int iResult = sqlite_exec(m_pSQL, qsSQL, SQLStaticCallback, this, 0);
  m_eMode = NORMAL;
  if (iResult != 0)
    throw new QSQLiteException(iResult);
	return m_iCount;
}

void QSQLite::Update(QString qsTabName, QString qsSet, QString qsWhere)
{
	QString qsSQL("UPDATE ");
	qsSQL += qsTabName;
	qsSQL += " SET ";
	qsSQL += qsSet;
	qsSQL += " WHERE ";	
	qsSQL += qsWhere;
  int iResult = sqlite_exec(m_pSQL, qsSQL, SQLStaticCallback, this, 0);
  if (iResult != 0)
    throw new QSQLiteException(iResult);
}


bool QSQLite::IsInTable(QString qsTabName, QString qsWhere)
{
	m_bIsInTable = false;
	m_eMode = ISINTABLE;
	QString qsSQL = "select * from ";
	qsSQL += qsTabName;
	qsSQL += " where ";
	qsSQL += qsWhere;
	int iResult = sqlite_exec(m_pSQL, qsSQL, SQLStaticCallback, this, 0);
  m_eMode = NORMAL;
  if (iResult != 0)
    throw new QSQLiteException(iResult);
  return m_bIsInTable;
}

void QSQLite::CreateTable(QString qsTabName, QString gsColumns)
{
  QString qsSQL = "create table ";
  qsSQL += qsTabName;
  qsSQL += " (";
  qsSQL += gsColumns;
  qsSQL += ");";
  int iResult = sqlite_exec(m_pSQL, qsSQL, SQLStaticCallback, this, 0);
  if (iResult != 0)
    throw new QSQLiteException(iResult);
}

