/***************************************************************************
                          qsqlite.h  -  description
                             -------------------
    begin                : Sat Feb 16 2002
    copyright            : (C) 2002 by Bernd Lachner
    email                : dev@lachner-net.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QSQLITE_H
#define QSQLITE_H

#include <qobject.h>
#include <qstring.h>
#include <qstringlist.h>


#include <sqlite.h>


/**Exception class for QSQLite
  *@author Bernd Lachner
  */
class QSQLiteException
{
  private:
    int m_iSQLiteErrorCode;
    QString m_qsMessage;
  public:
    /**Constructor
      *@param iSQLiteErrorCode Error code of the sqlite_exec function.
      */
    QSQLiteException(int iSQLiteErrorCode);
    /**Constructor
      *@param qsMessage Error message.
      */
    QSQLiteException(QString qsMessage);
    /**Returns the a message with the error description.
      *@return error description.
      */
    QString getMessage();
};


/**Class for accessing SQLite databases
  *@author Bernd Lachner
  */
class QSQLite : public QObject
{
  Q_OBJECT
signals:
  void RowData(int iColumns, char** aszData, char** aszColumnNames);
private:
  enum Mode { NORMAL, COUNT, ISINTABLE};
  Mode m_eMode;
  sqlite *m_pSQL;
  static int SQLStaticCallback(void *pObject, int iColumns, char** aszData, char** aszColumnNames);
  int SQLCallback(int iColumns, char** aszData, char** aszColumnNames);
  int m_iCount;
  bool m_bIsInTable;
public:
  /**Constructor. Opens a SQLite database
    *@param qsDatabaseName Name of the database file.
    */
  QSQLite(QString qsDatabaseName);
  /**Destructor. Closes the SQLite database.
    */
  ~QSQLite();
  /**Opens a SQLite database
    *@param qsDatabaseName Name of the database file.
    */
  void Open(QString qsDatabaseName);
  /**Closes a SQLite database
   */
  void Close();
  /**Creates a new table.
    *@param qsTabName Name of the table.
    *@param qsColumns Column names seperated by coma. Optionally with data types.
    */
  void CreateTable(QString qsTabName, QString gsColumns);
  /**Generates a select command.
    *@param poReceiver Object which should receive the data.
    *@param pszMember Slot for receiving the data. Must be defined with the SLOT macro.
    *@param qsTabName Name(s) of the table(s). The table names are coma separated.
    *@param qsColumns Names of the columns which should be selected. Coma separated. Default value: "*" (all columns).
    *@param qsWhere Optional 'where' to select the rows. Default value "" (all rows).
    */
	void Select(QObject *poReceiver, const char *pszMember, QString qsTabName, QString qsColumns = "*", QString qsWhere = "", QString qsOrderColumn = "", bool bDesc = false);
  void SelectDistinct(QObject *poReceiver, const char *pszMember, QString qsTabName, QString qsColumns = "*", QString qsWhere = "");
	void Insert(QString qsTabName, QStringList &qslistValues, QString qsValues = "");
	void Update(QString qsTabName, QString qsSet, QString qsWhere);
	void Delete(QString qsTabName, QString qsColumn, QString qsValue);
	void DeleteAll(QString qsTabName);
	void DeleteIn(QString qsTabName, QString qsColumn, QString qsSubSelect);
	void Delete(QString qsTabName, QString qsColumn1, QString qsValue1, QString qsColumn2, QString qsValue2, QString qsCombination = "AND");
	static QString MakeCharVal(QString qsVal);
	void ExecSQL(QString qsSQL);
	int Count(QString qsTabName, QString qsColumn, QString qsValue);
	bool IsInTable(QString qsTabName, QString qsWhere);
};

#endif
