/***************************************************************************
                          zshopigoodsview.cpp  -  description
                             -------------------
    begin                : Son Jul 20 2003
    copyright            : (C) 2003 by Bernd Lachner
    email                : dev@lachner-net.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlistview.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qmessagebox.h>
#include <qpushbutton.h>

#include "qlistviewextitem.h"
#include "qlistboxtextext.h"
#include "zshopigoodsview.h"
#include "zshopiaddnewgood.h"
#include "zshopiaddnewamountgroup.h"


ZShopiGoodsView::ZShopiGoodsView(QSQLite *poSQLite, QWidget *parent)
: ZShopiGoodsViewDesign(parent),
  m_poSQLite(poSQLite)
{
  RefreshGoodsList();
  RefreshGoodsGroupsList();
  RefreshShopsList();
  RefreshAmountGroupsList();
}

ZShopiGoodsView::~ZShopiGoodsView()
{
}

void ZShopiGoodsView::slotTabChanged(QWidget*)
{
  slotShopSelected();
  slotGoodGroupSelected();
}

void ZShopiGoodsView::RefreshGoodsList()
{
  //Show the goods
  ListViewGoodsList->clear();
  m_poSQLite->Select(this, SLOT(RowDataGoodsList(int, char**, char**)), "goods, category, amountgroups", "goods.id, goods.name, category.name, amountgroups.name" , "goods.category = category.id and goods.amountgroup = amountgroups.id");
}

void ZShopiGoodsView::RefreshGoodsGroupsList()
{
  //Show the good-groups 
  ListBoxGoodsGroups->clear();
  m_poSQLite->Select(this, SLOT(RowDataGoodsGroupsList(int, char**, char**)), "category","*", "", "name");
  slotGoodGroupSelected();  
}

void ZShopiGoodsView::RefreshShopsList()
{
  //Show the shops
  ListBoxShops->clear();
  m_poSQLite->Select(this, SLOT(RowDataShopList(int, char**, char**)), "shops","*", "", "name");
  slotShopSelected();
}


//slots and functions for the different tabs ************************

//Goods-Tab**********************************************************

void ZShopiGoodsView::slotDeleteGood()
{
  QListViewExtItem *poListViewItem = static_cast<QListViewExtItem*>(ListViewGoodsList->selectedItem());

  if (m_poSQLite->IsInTable("cart", "goodid = " + m_poSQLite->MakeCharVal(poListViewItem->getID())))
  {
    QMessageBox::information( this, tr("Can't delete good!"), tr(" The good you want to \n delete is still in use! \n So it can't be deleted."));
  }
  else
  {
    if (poListViewItem != NULL)
    {
      m_poSQLite->Delete("goods", "ID", poListViewItem->getID());
      delete poListViewItem;
      PushButtonDeleteGood->setEnabled(FALSE);
      PushButtonEditGood->setEnabled(FALSE);
    }
  }
}

void ZShopiGoodsView::slotEditGood()
{
  QListViewExtItem *poListViewItem = static_cast<QListViewExtItem*>(ListViewGoodsList->selectedItem());
  ZShopiAddNewGood oZShopiAddNewGood(m_poSQLite, poListViewItem->getID(), this, "ZShopiAddNewGood", TRUE);
  oZShopiAddNewGood.exec();
  RefreshGoodsList();
  
  PushButtonDeleteGood->setEnabled(FALSE);
  PushButtonEditGood->setEnabled(FALSE);
}

void ZShopiGoodsView::slotAddGood()
{
  ZShopiAddNewGood oZShopiAddNewGood(m_poSQLite, this, "ZShopiAddNewGood", TRUE);
  oZShopiAddNewGood.exec();
  RefreshGoodsList();
  
  PushButtonDeleteGood->setEnabled(FALSE);
  PushButtonEditGood->setEnabled(FALSE);  
}

void ZShopiGoodsView::RowDataGoodsList(int iColumns, char** aszData, char** aszColumnNames)
{
  QListViewExtItem *oListField = new QListViewExtItem(ListViewGoodsList);
  oListField->setID(aszData[0]);
  oListField->setText(0, aszData[1]);
  oListField->setText(1, aszData[2]);
  oListField->setText(2, aszData[3]);
}

void ZShopiGoodsView::slotGoodSelected()
{
  PushButtonDeleteGood->setEnabled(TRUE);
  PushButtonEditGood->setEnabled(TRUE);
}

//GoodsGroup-Tab*****************************************************

void ZShopiGoodsView::slotAddGoodsGroup()
{
  if (m_qsEditGoodsGroupID.isNull())
  {
    QStringList qslistNewData;
    qslistNewData.append(LineEditAddGoodsGroup->text());
    m_poSQLite->Insert("category",qslistNewData,"name");
  }
  else
  {
    QString qsSet = "name = " + QSQLite::MakeCharVal( LineEditAddGoodsGroup->text());
    m_poSQLite->Update("category", qsSet ,"id = " + m_qsEditGoodsGroupID );
    m_qsEditGoodsGroupID = QString::null;
    RefreshGoodsList();
    PushButtonGoodsGroup->setText(tr("Add"));
  }
  RefreshGoodsGroupsList();
  LineEditAddGoodsGroup->setText("");
}

void ZShopiGoodsView::slotDeleteGoodsGroup()
{
  QListBoxTextExt *poListItem = static_cast<QListBoxTextExt*>(ListBoxGoodsGroups->item(ListBoxGoodsGroups->currentItem()));
  
  if (m_poSQLite->IsInTable("goods", "category = " + m_poSQLite->MakeCharVal(poListItem->getID())))
  {
    QMessageBox::information( this, tr("Can't delete Good-Group!"), tr(" The Group you want to \n delete is still in use! \n So it can't be deleted."));
  }
  else
  {
    if (poListItem != NULL)
    {
      m_poSQLite->Delete("category","ID", poListItem->getID());
      delete poListItem;
    }
  }
}

void ZShopiGoodsView::slotEditGoodsGroup()
{
  QListBoxTextExt *poListItem = static_cast<QListBoxTextExt*>(ListBoxGoodsGroups->item(ListBoxGoodsGroups->currentItem()));

  PushButtonEditGoodsGroup->setEnabled(FALSE);
  PushButtonDeleteGoodsGroup->setEnabled(FALSE);
  PushButtonGoodsGroup->setEnabled(TRUE);
  PushButtonGoodsGroup->setText(tr("Accept"));

  LineEditAddGoodsGroup->setText(poListItem->text());
  m_qsEditGoodsGroupID = poListItem->getID();
}

void ZShopiGoodsView::RowDataGoodsGroupsList(int iColumns, char** aszData, char** aszColumnNames)
{
  QListBoxTextExt *poNewItem = new QListBoxTextExt(ListBoxGoodsGroups, aszData[1]);
  poNewItem->setID(aszData[0]);
}

void ZShopiGoodsView::slotTextNewGoodGroupChanged(const QString &qsText)
{
  if (qsText == "")
  {
    PushButtonGoodsGroup->setEnabled(FALSE);
  }
  else
  {
    PushButtonGoodsGroup->setEnabled(TRUE);
  }
}

void ZShopiGoodsView::slotGoodGroupSelected()
{
  if (ListBoxGoodsGroups->currentItem() == -1)
  {
    PushButtonDeleteGoodsGroup->setEnabled(false);
    PushButtonEditGoodsGroup->setEnabled(false);
  }
  else
  {
    PushButtonDeleteGoodsGroup->setEnabled(true);
    PushButtonEditGoodsGroup->setEnabled(true);
  }  
  PushButtonGoodsGroup->setText(tr("Add"));
  m_qsEditGoodsGroupID = QString::null;
  LineEditAddGoodsGroup->setText("");
  
}

//Shop-Tab***********************************************************
void ZShopiGoodsView::RowDataShopList(int iColumns, char** aszData, char** aszColumnNames)
{
  QListBoxTextExt *poNewItem = new QListBoxTextExt(ListBoxShops, aszData[1]);
  poNewItem->setID(aszData[0]);
}

void ZShopiGoodsView::slotDeleteShop()
{
  QListBoxTextExt *poListItem = static_cast<QListBoxTextExt*>(ListBoxShops->item(ListBoxShops->currentItem()));

  if (m_poSQLite->IsInTable("cart", "shopid = " + m_poSQLite->MakeCharVal(poListItem->getID())))
  {
    QMessageBox::information( this, tr("Can't delete shop!"), tr("The shop you want to \n delete is still in use! \n So it can't be deleted."));
  }
  else
  {
    if (poListItem != NULL)
    {
      m_poSQLite->Delete("shops","ID", poListItem->getID());
      delete poListItem;
    }
  }
}

void ZShopiGoodsView::slotEditShop()
{
  QListBoxTextExt *poListItem = static_cast<QListBoxTextExt*>(ListBoxShops->item(ListBoxShops->currentItem()));

  PushButtonEditShop->setEnabled(FALSE);
  PushButtonDeleteShop->setEnabled(FALSE);
  PushButtonAddShop->setEnabled(TRUE);
  PushButtonAddShop->setText(tr("Accept"));

  LineEditAddShop->setText(poListItem->text());
  m_qsEditShopID = poListItem->getID();
  
}

void ZShopiGoodsView::slotAddShop()
{
  if (m_qsEditShopID.isNull())
  {
    QStringList qslistNewData;
    qslistNewData.append(LineEditAddShop->text());
    m_poSQLite->Insert("shops",qslistNewData,"name");
  }
  else
  {
    QString qsSet = "name = " + QSQLite::MakeCharVal( LineEditAddShop->text());
    m_poSQLite->Update("shops", qsSet ,"id = " + m_qsEditShopID );
    m_qsEditShopID = QString::null;
    PushButtonAddShop->setText(tr("Add"));
  }
  RefreshShopsList();
  LineEditAddShop->setText("");
}

void ZShopiGoodsView::slotTextNewShopChanged(const QString &qsText)
{
  if (qsText == "")
  {
    PushButtonAddShop->setEnabled(FALSE);
  }
  else
  {
    PushButtonAddShop->setEnabled(TRUE);
  }
}

void ZShopiGoodsView::slotShopSelected()
{
  if (ListBoxShops->currentItem() == -1)
  {
    PushButtonDeleteShop->setEnabled(false);
    PushButtonEditShop->setEnabled(false);
  }
  else
  {
    PushButtonDeleteShop->setEnabled(true);
    PushButtonEditShop->setEnabled(true);
  }
  
  PushButtonAddShop->setText(tr("Add"));
  m_qsEditShopID = QString::null;
  LineEditAddShop->setText("");

}

//AmountGroup-Tab****************************************************

void ZShopiGoodsView::slotAddAmountGroup()
{
  ZShopiAddNewAmountGroup oZShopiAddNewAmountGroup(m_poSQLite, this, "ZShopiAddNewAmountGroup", TRUE);
  oZShopiAddNewAmountGroup.exec();
  RefreshAmountGroupsList();

  PushButtonDeleteAmountGroup->setEnabled(FALSE);
  PushButtonEditAmountGroup->setEnabled(FALSE);
}

void ZShopiGoodsView::RefreshAmountGroupsList()
{
  ListViewAmountGroups->clear();
  m_poSQLite->Select(this, SLOT(RowDataAmountGroups(int, char**, char**)), "amountgroups","*", "", "name");
}

void ZShopiGoodsView::RowDataAmountGroups(int iColumns, char** aszData, char** aszColumnNames)
{
  QListViewExtItem *oListField = new QListViewExtItem(ListViewAmountGroups);
  oListField->setID(aszData[0]);
  oListField->setText(0, aszData[1]);
  oListField->setText(1, aszData[2]);
  oListField->setText(2, aszData[3]);
}


void ZShopiGoodsView::slotAmountGroupSelected()
{
  PushButtonDeleteAmountGroup->setEnabled(TRUE);
  PushButtonEditAmountGroup->setEnabled(TRUE);
}

void ZShopiGoodsView::slotDeleteAmountGroup()
{
  QListViewExtItem *poListViewItem = static_cast<QListViewExtItem*>(ListViewAmountGroups->selectedItem());

  if (m_poSQLite->IsInTable("goods", "category = " + m_poSQLite->MakeCharVal(poListViewItem->getID())))
  {
    QMessageBox::information( this, tr("Can't delete group!"), tr(" The group you want to \n delete is still in use! \n So it can't be deleted."));
  }
  else
  {
    if (poListViewItem != NULL)
    {
      m_poSQLite->Delete("amountgroups", "ID", poListViewItem->getID());
      delete poListViewItem;
      PushButtonDeleteAmountGroup->setEnabled(FALSE);
    }
  }
}

void ZShopiGoodsView::slotEditAmountGroup()
{
  QListViewExtItem *poListViewItem = static_cast<QListViewExtItem*>(ListViewAmountGroups->selectedItem());

  ZShopiAddNewAmountGroup oZShopiAddNewAmountGroup(m_poSQLite, poListViewItem->getID(), this, "ZShopiAddNewAmountGroup", TRUE);
  oZShopiAddNewAmountGroup.exec();
  RefreshAmountGroupsList();
  RefreshGoodsList();

  PushButtonDeleteAmountGroup->setEnabled(FALSE);
  PushButtonEditAmountGroup->setEnabled(FALSE);
}

void ZShopiGoodsView::Refresh()
{
  RefreshGoodsList();
  RefreshGoodsGroupsList();
  RefreshShopsList();
  RefreshAmountGroupsList();
}
