/***************************************************************************
                          zshopipreferenceview.cpp  -  description
                             -------------------
    begin                : Son Nov 9 2003
    copyright            : (C) 2003 by Bernd Lachner
    email                : dev@lachner-net.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "zshopipreferenceview.h"

#include <qfile.h>
#include <qmessagebox.h>

#ifdef QTOPIA
#include "scqtfiledialog.h"
#else
#include <qfiledialog.h>
#endif

ZShopiPreferenceView::ZShopiPreferenceView(QSQLite *poSQLite, QWidget *parent)
: m_poSQLite(poSQLite)
{
}

ZShopiPreferenceView::~ZShopiPreferenceView()
{
}

bool ZShopiPreferenceView::Copy(const QString &qsFrom, const QString &qsTo)
{
  if (qsFrom == qsTo)
  {
    // Do nothing, if source file is destinataion file
    return true;
  }
  QFile oFrom(qsFrom);
  QFile oTo(qsTo);
  // Open files
  bool bFromOpen = oFrom.open(IO_Raw | IO_ReadOnly);
  bool bToOpen = oTo.open(IO_Raw | IO_WriteOnly);
  // Copy files
  int iError = 0;
  if (bFromOpen && bToOpen)
  {
    char acBuffer[255];
    int iRead = oFrom.readBlock(acBuffer, 255);
    while (iRead > 0 && iError != -1)
    {
      iError = oTo.writeBlock(acBuffer, iRead);
      iRead = oFrom.readBlock(acBuffer, 255);
    }
  }
  // Close opened files
  if (bFromOpen)
  {
    oFrom.close();
  }
  if (bToOpen)
  {
    oTo.close();
  }
  if (iError == -1 || !bFromOpen || !bToOpen)
  {
    // Error
    return false;
  }
  else
  {
    // Copy successfull
    return true;
  }
  
}

QString ZShopiPreferenceView::GetTempFilename()
{
  QString qsResult = QString::null;
  for (unsigned long  i = 0; i < 0xffffffff; i++)
  {
    qsResult = QDir::home().absFilePath(QString("dbl")+QString::number(i)+".tmp");
    if (!QFile::exists(qsResult))
      break;
  }
  return qsResult;
}
  
void ZShopiPreferenceView::slotImportDatabase()
{
  QString qsFileDir;
#ifdef QTOPIA
  qsFileDir = ScQtFileDialog::getOpenFileName(QDir::homeDirPath(), "*.dbl", this );
#else
  qsFileDir = QFileDialog::getOpenFileName(QDir::homeDirPath(), "SQLite Database (*.dbl)", this );
#endif
  if (!qsFileDir.isEmpty())
  {                                                             
    if (QMessageBox::information( this, tr("Database import"), tr(" Your old data will be lost! Continue Import?"), tr("&Yes"), tr("&No")) == 0)
    {
      QString qsTempFilename = GetTempFilename();
      Copy(qsFileDir, qsTempFilename);
      try
      {
        QSQLite oSQLite(qsTempFilename);
        if (oSQLite.IsInTable("sqlite_master", "type = 'table' and name = 'goods'") &&
          oSQLite.IsInTable("sqlite_master", "type = 'table' and name = 'category'") &&
          oSQLite.IsInTable("sqlite_master", "type = 'table' and name = 'shops'") &&
          oSQLite.IsInTable("sqlite_master", "type = 'table' and name = 'cart'") &&
          oSQLite.IsInTable("sqlite_master", "type = 'table' and name = 'amountgroups'"))
        {
          oSQLite.Close();
          m_poSQLite->Close();
          QFile::remove(QDir::home().absFilePath("zshopi.dbl"));
          QDir oDir;
          oDir.rename(qsTempFilename, QDir::home().absFilePath("zshopi.dbl"));
          m_poSQLite->Open(QDir::home().absFilePath("zshopi.dbl"));
        }
        else
        {
          QMessageBox::critical( this, tr("Can't import database!"), tr(" The database is not a correct ZShopi database."));
        }
      }
      catch (...)
      {
        QMessageBox::critical( this, tr("Can't import database!"), tr(" The file is not ZShopi/SQLite database."));
      }
      QFile::remove(qsTempFilename);
    }
  }
}                                                                   

void ZShopiPreferenceView::slotExportDatabase()
{
  QString qsFileDir;
#ifdef QTOPIA
  qsFileDir = ScQtFileDialog::getSaveFileName(QDir::homeDirPath(), "*.dbl", this );
#else
  qsFileDir = QFileDialog::getSaveFileName(QDir::homeDirPath(), "SQLite Database (*.dbl)", this );
#endif
  if (!qsFileDir.isEmpty())
  {
    // Check if file exit and ask for overwrite if necessary
    bool bExport = !QFile::exists(qsFileDir);
    if (!bExport)
    {
      if (QMessageBox::information( this, tr("Database export"), tr(" File exists! Overwrite?"), tr("&Yes"), tr("&No")) == 0)
        bExport = true;
    }
    // Export (copy) database file
    if (bExport)
    {
      m_poSQLite->Close();
      Copy(QDir::home().absFilePath("zshopi.dbl"), qsFileDir);
      m_poSQLite->Open(QDir::home().absFilePath("zshopi.dbl"));
    }
  }
}

